slurm_is_working = False

papi_is_working = False

nvidia_is_working = False

network_is_working = False

disk_is_working = False

cpu_is_working = False

memory_is_working = False


flask_path_reg = False