# General commands 1-10000

finalize = 1


# Job commands 10001-20000



# CPU commands 20001-30000

meassure_cpu_load = 20001
retrieve_cpu_load = 20002

meassure_cpu_frequency = 20011


# GPU commands 30001-40000

meassure_gpu_load = 30001
retrieve_gpu_load = 30002

meassure_gpu_frequency = 30011


# Network commands 40001-50000



# IO commands 50001-60000



# General application commands 60001-70000



# Alya commands 70001-80000